package edu.georgetown.gucs.dictionary;

import edu.georgetown.gucs.utility.ExceptionHandler;
import edu.georgetown.gucs.utility.Global;
import edu.georgetown.gucs.utility.SdTextException;

/**
 * Prints statistics for a {@linkplain Dictionary}:
 * <ul>
 * <li>Name</li>
 * <li>Language</li>
 * <li>Number of documents</li>
 * <li>Number of tokens</li>
 * <li>If the dictionary has been trimmed (including the IDF range, if trimmed)</li>
 * <li>Minimum and maximum IDF</li>
 * <li>List of tokenizers</li>
 * </ul>
 *
 * @author Clay Shields
 */
public class ShowDictionaryStatistics {

    private static Dictionary dictionary;

    /**
     * Prints the dictionary statistics for the specified dictionary
     *
     * @param dictionary_file the string filename containing the serialized dictionary
     */
    public static void print(String dictionary_file) throws SdTextException {
        try {
            dictionary = new Dictionary(dictionary_file);
            dictionary.showStatistics();
        } catch (SdTextException sdtextE) {
            throw sdtextE;
        } catch (Exception e) {
            ExceptionHandler.handleException(e);
        }
    }

    /**
     * Provides the dictionary statistics for the specified dictionary
     *
     * @param dictionary_file the string filename containing the serialized dictionary
     * @return the string containing the specified dictionary's statistics
     */
    public static String getStatistics(String dictionary_file) throws SdTextException {
        try {
            dictionary = new Dictionary(dictionary_file);
            return dictionary.getStatistics();
        } catch (SdTextException sdtextE) {
            throw sdtextE;
        } catch (Exception e) {
            ExceptionHandler.handleException(e);
        }
        return "";
    }


}
