package edu.georgetown.gucs.fingerprinter;

import java.io.File;
import java.io.IOException;
import java.util.List;
import java.util.ArrayList;


import edu.georgetown.gucs.dictionary.Dictionary;
import edu.georgetown.gucs.tokenizers.TokenizerList;
import edu.georgetown.gucs.utility.*;


/**
 * A thread worker for doing n^2 matching over all files in a
 * directory.
 * <p>
 * Based in  part on code from this answer:
 * http://stackoverflow.com/questions/8974638/blocking-queue-and-multi-threaded-consumer-how-to-know-when-to-stop
 *
 * @author Clay Shields
 */

public class BulkFingerprinterWorker extends Thread {

    private Fingerprinter fingerprinter;
    private List<Fingerprint> results;
    private FileLister fileProducer;
    private File poisonPill;
    private BulkFingerprinter parent;
    private TokenizerList tokenizer;


    // Takes a file and a dictionary and refernce to a Map to add the fp to
    // Creates a fingerprint
    // Returns the file name and resulting fingerprint to the parent

    public BulkFingerprinterWorker(String fingerprinterName, Dictionary dictionary, BulkFingerprinter parent,
                                   FileLister fileProducer, File poison) throws SdTextException {

        try {
            this.fingerprinter = (Fingerprinter) ObjectBuilder.buildObject(fingerprinterName);
            this.fingerprinter.setDictionary(dictionary);
            tokenizer = dictionary.getTokenizers();
            this.parent = parent;
            this.fileProducer = fileProducer;
            this.poisonPill = poison;

            results = new ArrayList<>();
        } catch (SdTextException sdTextE) {
            throw sdTextE;
        } catch (Exception e) {
            ExceptionHandler.handleException(e);
        }
    }

    @Override
    public void run() {
        try {
            while (true) {

                if (isInterrupted()) {
                    return;
                }

                File file = fileProducer.takeFile();
                if (file != poisonPill) {
                    List<Fingerprint> fp = new ArrayList<>();

                    fp.addAll(fingerprinter.computeFingerprint(file.getPath()));

                    parent.putResults(fp);

                } else {

                    fileProducer.putPoison(poisonPill);
                    break;
                }
            }

        } catch (SdTextException sdtext) {
            Thread.currentThread().interrupt();//preserve the message
            return;
        }
    }

}