package edu.georgetown.gucs.monitor;

import edu.georgetown.gucs.utility.SdTextException;
import org.jdom.Document;
import org.jdom.Element;
import org.jdom.JDOMException;
import org.jdom.input.SAXBuilder;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Optional;

/**
 * @author Evan B.
 * @since 5/25/15.
 */
public class MonitorConfigParser {

    /**
     * local field variables
     */
    String configFilename;
    Optional<String> defaultFingerprinter;
    Optional<String> defaultDictionary;
    Optional<String> defaultOutput;
    List<Optional<String>> defaultRegex = new ArrayList<>();

    /**
     * part of the builder pattern
     */
    Optional<String> whitelistFingerprinter;
    Optional<String> whitelistDictionary;
    Optional<List<String>> whitelistDirectories;
    Optional<List<String>> whitelistFiles;
    Optional<String> whitelistOutput;

    /**
     * Constructor that reads an XML configuration file
     *
     * @param configFilename the path to a monitor configuration file
     */
    public MonitorConfigParser(String configFilename) throws JDOMException, SdTextException, IOException {
        this.configFilename = configFilename;
        parseConfiguration();
    }

    private MonitorConfigParser(WhiteListBuilder whitelist) {
        this.whitelistFingerprinter = whitelist.whitelistFingerprinter;
        this.whitelistDictionary = whitelist.whitelistDictionary;
        this.whitelistDirectories = whitelist.whitelistDirectories;
        this.whitelistFiles = whitelist.whitelistFiles;
        this.whitelistOutput = whitelist.whitelistOutput;
    }

    /**
     * Create a new class called the monitor class, or something similar.
     * It should take an configuration file. The configuration file should include
     * a dictionary, a fingerprinter, a list of directories to be monitored,
     * and a list of file names to be excluded from monitoring.
     * It should specify a directory for output of fingerprint files as well.
     */
    private void parseConfiguration() throws SdTextException, JDOMException, IOException {

        File xmlConfig = new File(configFilename);
        SAXBuilder builder = new SAXBuilder();

        Document doc = builder.build(xmlConfig);
        Element monitor = doc.getRootElement();
        getDefaults(monitor);

        getWhiteList(monitor.getChild("whitelist").getChildren());

    }


    private void getDefaults(Element defaults) {

        if (!defaults.getChildren("default").isEmpty()) {

            Element defaultsChildren = defaults.getChild("default");

            defaultDictionary = Optional.ofNullable(defaultsChildren.getChildText("dictionary"));
            defaultFingerprinter = Optional.ofNullable(defaultsChildren.getChildText("fingerprinter"));
            defaultOutput = Optional.ofNullable(defaultsChildren.getChildText("output"));


            for (Element e : (List<Element>) defaultsChildren.getChild("ignore").getChildren()) {

                defaultRegex.add(Optional.ofNullable(e.getAttributeValue("regex")));

            }

        } else {
            System.out.println("error");
        }
    }

    private void getWhiteList(List<Element> whitelist) {

        for (Element e : whitelist) {

            e.getAttributeValue("fingerprinter");
            e.getAttributeValue("dictionary");

            for (Element directory : (List<Element>) e.getChildren("directory")) {
                System.out.println(directory.getAttributeValue("path"));
            }

            for (Element directory : (List<Element>) e.getChildren("file")) {
                System.out.println(directory.getAttributeValue("path"));
            }

            System.out.println(e.getChild("output").getAttributeValue("path") + "\n");
        }


    }

    public void testWhiteList() {

    }

    public void testDefaults() {

        defaultOutput.ifPresent(System.out::println);
        defaultFingerprinter.ifPresent(System.out::println);
        defaultDictionary.ifPresent(System.out::println);

        for (Optional<String> str : defaultRegex) {

            str.ifPresent(System.out::println);
        }

    }


    private void getBlacklist(List<Element> blackList) {


    }


    public Optional<String> getFingerprinter(){ return whitelistFingerprinter; }
    public Optional<String> getDictionary(){return whitelistDictionary; }

    public static class WhiteListBuilder {

        private Optional<String> whitelistFingerprinter;
        private Optional<String> whitelistDictionary;
        private Optional<List<String>> whitelistDirectories;
        private Optional<List<String>> whitelistFiles;
        private Optional<String> whitelistOutput;

        public WhiteListBuilder Dictionary(String fingerprinter) {

            whitelistDictionary = Optional.ofNullable(fingerprinter);
            return this;
        }

        public WhiteListBuilder fingerprinter(String fingerprinter) {

            whitelistFingerprinter = Optional.ofNullable(fingerprinter);
            return this;
        }

        public WhiteListBuilder directories(List<String> directory) {

            whitelistDirectories = Optional.ofNullable(directory);
            return this;
        }


        public WhiteListBuilder files(List<String> files) {
            whitelistFiles = Optional.ofNullable(files);
            return this;
        }

        public WhiteListBuilder output(String output) {

            whitelistOutput = Optional.ofNullable(output);
            return this;
        }

        public MonitorConfigParser build() {

            return new MonitorConfigParser(this);

        }

    }
}