package edu.georgetown.gucs.tokenizers;

import java.util.Collections;
import java.util.List;
import java.util.Vector;

import edu.georgetown.gucs.utility.ExceptionHandler;
import edu.georgetown.gucs.utility.SdTextException;
import edu.georgetown.gucs.utility.Token;

/**
 * Eliminates tokens that are longer than a given length
 *
 * @author Clay Shields
 * @author Lindsay Neubauer
 */
public class MaximumLengthTokenizer extends Tokenizer {

    private static final long serialVersionUID = -1057327977380694005L;
    private final int maxTokenLength;

    /**
     * Constructor that sets the maximum token length to be considered
     *
     * @param length the string value of the maximum token length to be included
     */
    public MaximumLengthTokenizer(String length) {
        this.maxTokenLength = Integer.parseInt(length);
    }

    /**
     * Eliminates tokens that are longer than the length specified in the constructor
     */
    @Override
    public List<Token> tokenize(List<Token> tokens) throws SdTextException {
        List<Token> tokenVector = Collections.synchronizedList(new Vector<Token>());
        try {
            synchronized (tokenVector) {
                for (Token token : tokens) {
                    if (token.name().length() <= this.maxTokenLength) {
                        tokenVector.add(token);
                    }
                }
                return tokenVector;
            }
        } catch (Exception e) {
            ExceptionHandler.handleException(e);
        }
        return tokenVector;
    }
}
