package edu.georgetown.gucs.tokenizers;

import java.util.List;
import java.util.Vector;

/**
 * Provides the tokenizer parameters, including tokenizers, manglers and splitters
 *
 * @author Lindsay Neubauer
 */
public class TokenizerParameters {

    private List<String> tokenizers = new Vector<String>();
    private List<String> manglers = new Vector<String>();
    private List<String> splitters = new Vector<String>();

    /**
     * Constructor that sets the tokenizer, mangler, and splitter vectors
     *
     * @param tokenizers
     *          the vector of tokenizer strings
     * @param manglers
     *          the vector of mangler strings
     * @param splitters
     *          the vector of splitter strings
     */
    public TokenizerParameters(Vector<String> tokenizers, Vector<String> manglers, Vector<String> splitters) {
        this.tokenizers = tokenizers;
        this.manglers = manglers;
        this.splitters = splitters;
    }

    /**
     * Provides the tokenizers
     *
     * @return the vector of tokenizer strings
     */
    public List<String> tokenizers() {
        return this.tokenizers;
    }

    /**
     * Provides the manglers
     *
     * @return the vector of mangler strings
     */
    public List<String> manglers() {
        return this.manglers;
    }

    /**
     * Provides the splitters
     *
     * @return the vector of splitter strings
     */
    public List<String> splitters() {
        return this.splitters;
    }
}
