package edu.georgetown.gucs.utility;

import java.io.File;
import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.RecursiveTask;

/**
 * @author Evan B.
 * @since 09/12/14
 */
public class FileListerWorker extends RecursiveTask<ArrayList<File>> {


    private final String directoryName;

    /**
     * @param directoryName folder to traverse
     */
    FileListerWorker(String directoryName) {

        this.directoryName = directoryName;

    }

    @Override
    protected ArrayList<File> compute() {

        ArrayList<File> fileList = new ArrayList<>();
        File directory = new File(directoryName);
        File[] fList = directory.listFiles();
        List<FileListerWorker> tasks = new ArrayList<>();

        if (fList != null) {

            for (File file : fList) {
                // ignore .DS_Store files
                // ignore .svn files
                // ignore .git files
                if (!file.toString().contains(".DS_Store") ||
                        !(file.toString().contains(".svn")) ||
                        !(file.toString().contains(".git"))) {

                    if (file.isDirectory()) {
                        FileListerWorker task = new FileListerWorker(file.getAbsolutePath());
                        task.fork();
                        tasks.add(task);
                    } else {
                        fileList.add(file);
                    }

                }

            }
        }

        addResultsFromTasks(fileList, tasks);

        return fileList;
    }

    //For each task stored in the list of tasks, call the join() method that will wait for its finalization and then will return the result of the task.
    //Add that result to the list of strings using the addAll() method.
    private void addResultsFromTasks(List<File> fileList, List<FileListerWorker> tasks)
    {
        for (FileListerWorker item : tasks)
        {
            fileList.addAll(item.join());
        }
    }
}


