package edu.georgetown.gucs.utility;

import com.lowagie.text.*;
import com.lowagie.text.pdf.DefaultFontMapper;
import com.lowagie.text.pdf.PdfContentByte;
import com.lowagie.text.pdf.PdfTemplate;
import com.lowagie.text.pdf.PdfWriter;
import org.jfree.chart.ChartColor;
import org.jfree.chart.ChartUtilities;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.axis.NumberAxis;
import org.jfree.chart.plot.PlotOrientation;
import org.jfree.chart.plot.XYPlot;
import org.jfree.chart.renderer.xy.XYErrorRenderer;
import org.jfree.data.xy.*;
import org.jfree.util.ShapeUtilities;

import javax.swing.*;
import java.awt.*;
import java.awt.geom.Rectangle2D;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.*;
import java.util.List;


/**
 * @author Evan B.
 * @since 10/02/14
 * Implements graph capabilities
 * provided by: jfree.org
 */
public class JGraph {

    private String fileName;
    private String title = "";
    private HashMap<String, Pair<Integer, Double>> dataSet;

    private final static int BLACK_WHITE = 100;
    private final static int COLOR = 101;


    private Map<String, List<Pair<Double, XYDataItem>>> dataItemList = new HashMap<>();

    private final static Color[] GRAY_SCALE = new Color[]{Color.BLACK, Color.GRAY, Color.DARK_GRAY};

    private final static Color[] DARK_COLORS = new Color[]{Color.DARK_GRAY, ChartColor.DARK_RED, ChartColor.DARK_BLUE,
            ChartColor.DARK_GREEN, ChartColor.DARK_MAGENTA, ChartColor.DARK_CYAN,};

    private final static Shape[] SHAPES = new Shape[]{ShapeUtilities.createRegularCross(3, 1),
            ShapeUtilities.createDiagonalCross(3, 1), ShapeUtilities.createUpTriangle(3),
            ShapeUtilities.createDownTriangle(3), ShapeUtilities.createDiamond(3)};


    /**
     * @param dataSet the mangler & its average matcher score
     */
    public JGraph(HashMap<String, Pair<Integer, Double>> dataSet) {

        this.dataSet = dataSet;
        this.fileName = "generateRandomFilename";

    }

    /**
     * @param dataSet  the mangler & its average matcher score
     * @param fileName the name of a file to save graph
     */
    public JGraph(HashMap<String, Pair<Integer, Double>> dataSet, String fileName) {

        this.dataSet = dataSet;
        this.fileName = fileName;

    }

    public void setTitle(String title) {

        this.title = title;

    }

    /**
     * Maps a each mangler to its coordinates
     */
    private void createPlots() {

        int percent;
        String lastMangler = "";

        // For each mangler, find percent deleted and the score
        for (Map.Entry<String, Pair<Integer, Double>> set : dataSet.entrySet()) {


            //"IDF:" + sketchy?!
            String idfRange = set.getKey().substring(
                    set.getKey().lastIndexOf(":") + 1, set.getKey().length()) + " ";

            String mangler;
            //if it does not contain null
            // Global.debug("key", set.getKey());
            if (!set.getKey().contains("null")) {


                mangler = set.getKey().substring(0, set.getKey().lastIndexOf("-"));
                lastMangler = mangler;

                percent = Integer.parseInt(set.getKey().substring(set.getKey().lastIndexOf("-")
                        + 1, set.getKey().lastIndexOf(":")));
            } else {
                percent = 0;
                mangler = set.getKey();
                // Global.debug("lastMangler=",mangler);
            }

            //if it's the mangler
            if (!dataItemList.containsKey(idfRange)) {

                XYDataItem xyplots = new XYDataItem((double) percent, (double) set.getValue().getA());

                ArrayList<Pair<Double, XYDataItem>> pairList = new ArrayList<>();

                pairList.add(new Pair<>(set.getValue().getB(), xyplots));

                dataItemList.put(idfRange, pairList);

            } else {

                //Global.debug("Mangler", mangler);
                Pair<Double, XYDataItem> newPair = new Pair<>(set.getValue().getB(),
                        new XYDataItem((double) percent, (double) set.getValue().getA()));

                dataItemList.get(idfRange).add(newPair);

            }

            //temporary fix, find the null mangler
            //if the null mangler is not the only item in the hashmap
            //get its pairvalue, add it to the "mangler" key
            //remove the pairvalue from the list


        }


    }

    public void savePlot() throws SdTextException {

        try {
            createPlots();

            savePlot(fileName, title, "Percent Mangled", "Average Score", COLOR, createDataset(dataItemList));

        } catch (SdTextException sdtext) {
            throw sdtext;
        } catch (Exception e) {
            ExceptionHandler.handleException(e);
        }
    }

    /**
     * Writes a scatter plot to a PDF, PNG, or JPEG file
     *
     * @param outFilename the string name or path to store the resulting scatter plot; the extension of the filename is used to
     *                    determine the type of file to write, PDF is the default
     * @param title       the string title of the scatter plot
     * @param domainAxis  the string label for the domain axis
     * @param rangeAxis   the string label for the range axis
     * @param colorScheme the <code>int</code> color scheme value (<code>Global.BLACK_WHITE</code> or <code>Global.COLOR</code>)
     * @param dataset     the xyDataset containing the points to include in this scatter plot
     */
    private void savePlot(String outFilename, String title, String domainAxis, String rangeAxis, int colorScheme,
                          XYDataset dataset) throws SdTextException {
        try {
            // XYLineAndShapeRenderer renderer = new XYLineAndShapeRenderer(lines, true);
            XYErrorRenderer renderer = new XYErrorRenderer();
            renderer.setBaseLinesVisible(true);

            //calculate Confidence interval
            //renderer.setSeriesShape(0,);
            for (int i = 0; i < dataset.getSeriesCount(); i++) {
                if (colorScheme == BLACK_WHITE) {
                    renderer.setSeriesPaint(i, GRAY_SCALE[i % GRAY_SCALE.length]);
                } else {
                    renderer.setSeriesPaint(i, DARK_COLORS[i % DARK_COLORS.length]);
                }
                renderer.setSeriesShape(i, SHAPES[i % SHAPES.length]);


            }


            XYPlot plot = new XYPlot(dataset, new NumberAxis(domainAxis), new NumberAxis(rangeAxis), renderer);
            plot.setOrientation(PlotOrientation.VERTICAL);

            if (UIManager.get("Panel.background") == null) {
                UIManager.put("Panel.background", Color.WHITE);
            }

            //(dataset.getSeriesCount() >= 1)
            JFreeChart chart = new JFreeChart(title, JFreeChart.DEFAULT_TITLE_FONT, plot, true);

            int width = 640, height = 480;
            String outputFile = outFilename;
            if (outputFile.toLowerCase(Locale.ENGLISH).endsWith(".png")) {
                try {
                    ChartUtilities.saveChartAsPNG(new File(outputFile), chart, width, height);
                } catch (IOException e) {
                    ExceptionHandler.handleException(e);
                }
            } else if (outputFile.toLowerCase(Locale.ENGLISH).endsWith(".jpeg")) {
                try {
                    ChartUtilities.saveChartAsJPEG(new File(outputFile), chart, width, height);
                } catch (IOException e) {
                    ExceptionHandler.handleException(e);
                }
            } else {
                if (!outputFile.toLowerCase(Locale.ENGLISH).endsWith(".pdf")) {

                    outputFile += ".pdf";
                }

                // Code help from: http://jtoee.blogspot.com/2008/04/writing-pdf-charts-with-itext-in-less.html
                com.lowagie.text.Document document = new com.lowagie.text.Document(new com.lowagie.text.Rectangle(width, height));
                try {
                    // Overwrites a file if one with the same name already exists
                    PdfWriter pdfWriter = PdfWriter.getInstance(document, new FileOutputStream(outputFile));
                    document.open();
                    PdfContentByte pdfContentbyte = pdfWriter.getDirectContent();
                    PdfTemplate pdfTemplate = pdfContentbyte.createTemplate(width, height);
                    Graphics2D graphics = pdfTemplate.createGraphics(width, height, new DefaultFontMapper());

                    Rectangle2D rectangle = new Rectangle2D.Double(0, 0, width, height);
                    chart.draw(graphics, rectangle);

                    graphics.dispose();
                    pdfContentbyte.addTemplate(pdfTemplate, 0, 0);
                } catch (FileNotFoundException | DocumentException e) {
                    ExceptionHandler.handleException(e);
                }
                document.close();
            }
        } catch (SdTextException sdtextE) {
            throw sdtextE;
        } catch (Exception e) {
            ExceptionHandler.handleException(e);
        }
    }


    /**
     * Creates a dataset containing x and y coordinate data
     *
     * @param results the map containing the series labels and list of x and y coordinates to plot
     * @return the XYDataSet containing the series of x and y coordinate data
     */
    private XYDataset createDataset(Map<String, List<Pair<Double, XYDataItem>>> results) {
        XYIntervalSeriesCollection dataset = new XYIntervalSeriesCollection();

        for (Map.Entry<String, List<Pair<Double, XYDataItem>>> result : results.entrySet()) {

            XYIntervalSeries series = new XYIntervalSeries(result.getKey());

            //result.getValue().forEach(series::add);

            for (Pair<Double, XYDataItem> item : result.getValue()) {

                double confPos = item.getA() + item.getB().getYValue();
                double confNeg = item.getB().getYValue() - item.getA();

                //dataitem x = percent mangled
                //dataitem y = mean

                XYDataItem point = item.getB();

                /**
                 * Debug Statements
                 System.out.println("XVal: " + point.getXValue()
                 + " Yval: " + point.getYValue() +
                 " confPos: " + confPos + " confNeg " + confNeg);
                 **/
                series.add(point.getXValue(), point.getXValue(), point.getXValue(),
                        point.getYValue(), confNeg, confPos);

            }

            dataset.addSeries(series);
        }


        return dataset;
    }

}
