package edu.georgetown.gucs.utility;

import java.io.Serializable;
import java.util.Comparator;

/**
 * Creates an ordered pair
 *
 * @param <A> the type of the first object in this pair
 * @param <B> the type of the second object in this pair
 * @author Mark Maloof
 */
public class Pair<A, B> implements Serializable, Comparable<Pair> {

    private static final long serialVersionUID = -5801412717752975197L;
    private A a;
    private B b;

    /**
     * Constructor that sets the both objects in this pair
     *
     * @param a the A object to set as the first object of this pair
     * @param b the B object to set as the second object of this pair
     */
    public Pair(A a, B b) {
        setA(a);
        setB(b);
    }

    /**
     * Sets the first object in this pair
     *
     * @param a the first object of this pair
     */
    public void setA(A a) {
        this.a = a;
    }

    /**
     * Sets the second object in this pair
     *
     * @param b the second object of this pair
     */
    public void setB(B b) {
        this.b = b;
    }

    /**
     * Provides the first object in this pair
     *
     * @return the first object in this pair
     */
    public A getA() {
        return this.a;
    }

    /**
     * Provides the second object in this pair
     *
     * @return the second object in this pair
     */
    public B getB() {
        return this.b;
    }

    /**
     * Provides a string representation of the pair in the form (a, b)
     */
    @Override
    public String toString() {
        return ("(" + this.a + ", " + this.b + ")");
    }

    @Override
    public int compareTo(Pair o) {
        Token o1 = (Token) o.getA();
        Token o2 = (Token) a;
        return o1.compareTo(o2);
    }
}
