package sdtext;

import edu.georgetown.gucs.utility.Pair;
import edu.georgetown.gucs.utility.SdTextException;

import java.io.Serializable;
import java.util.*;

/**
 * @author Evan
 * @since 5/19/15.
 */
public class Results {

    private Map<String, Integer> resultMap = new HashMap<>();
    private ValueComparator scores = new ValueComparator(resultMap);
    private SortedMap<String, Integer> resultScores = Collections.synchronizedSortedMap(new TreeMap<String, Integer>(scores));

    private List<String> resultList = new ArrayList<>();
    private List<Pair<String, Integer>> resultPair = new ArrayList<>();

    /**
     * @param result a list of pair elements containing the 
     *               filename and score of a given comparison
     *               calls addResultPair to initialize result fields 
     */
    Results(List<Pair<String, Integer>> result) {
        this.resultPair = result;
        addResultPair();
    }

    /**
     * @param result a map of elements with the key as a filename and the value as its
     *               comparison score. Call's addResultMap to initialize result fields
     */
    Results(Map<String, Integer> result) {

        //this is where the error was [resultMap = result]
        resultMap.putAll(result);
        addResultMap();

    }

    /**
     * sorts a list of pair elements in descending order by score
     * stores pair results in a list of strings with each result in the 
     * format of filename:score 
     */
    private void addResultPair() {

        Collections.sort(resultPair, new HightoLowPair());

        for (Pair<String, Integer> r : resultPair) {

            resultMap.put(r.getA(), r.getB());
            resultList.add(r.getA() + ":" + r.getB());
        }

        resultScores.putAll(resultMap);

    }

    /**
     * Takes a map with filename keys and score values.
     * Sorts the information in descending order by score
     *
     */
    private void addResultMap() {

        for (Map.Entry<String, Integer> result : resultMap.entrySet()) {

            resultList.add(result.getKey() + ":" + result.getValue());
            resultMap.put(result.getKey(), result.getValue());

        }
        resultScores.putAll(resultMap);

    }

    /**
     * @return a map of scores with the key as the filename 
     * and the value as the score 
     */
    public Map<String, Integer> getResultMap() throws SdTextException {

        if(resultScores.isEmpty()) {
            throw new SdTextException("Results:getResultMap:no results to return");
        }

        return resultScores;
    }

    /**
     *
     * @return returns a list of matched scores in the format 
     * filename:score 
     */
    public List<String> getResultList() throws SdTextException {

        if(resultScores.isEmpty()) {
            throw new SdTextException("Results:getResultList:no results to return");
        }

        return resultList;
    }

    /**
     * prints the results of a comparison trial 
     */
    public void printResults() {

	if(resultScores.isEmpty()) {
	    System.out.println("No results found");
	}

	
	
        for (Map.Entry<String, Integer> result : resultScores.entrySet()) {

            System.out.println("fileName: " + result.getKey() + " score: " + result.getValue());

        }

    }

    /**
     * Comparator that orders score pairs highest to lowest
     */
    private static class HightoLowPair implements Comparator<Pair<String, Integer>>, Serializable {

        @Override
        public int compare(Pair<String, Integer> o1, Pair<String, Integer> o2) {

            //by using the second argument we get descending order
            return o2.getB().compareTo(o1.getB());
        }
    }

    /**
     * comparitor that orders maps highest to lowest 
     */
    class ValueComparator implements Comparator<String> {

        Map<String, Integer> base;

        public ValueComparator(Map<String, Integer> base) {
            this.base = base;
        }

        // Note: this comparator imposes orderings that are inconsistent with equals.
        public int compare(String a, String b) {
            if (base.get(a) >= base.get(b)) {
                return -1;
            } else {
                return 1;
            } // returning 0 would merge keys
        }
    }

}
